package de.hawhamburg.reachability.sensor.adapter;

import org.apache.log4j.Logger;

import com.google.gson.Gson;

import de.hawhamburg.reachability.ProjectInfo;
import de.hawhamburg.reachability.reasoner.enumeration.ReasonerDimension;
import de.hawhamburg.reachability.sensor.Sensor;
import de.hawhamburg.reachability.sensor.SensorAdapter;
import de.hawhamburg.reachability.sensor.SensorAdapterFactory;
import de.hawhamburg.reachability.sensor.SensorAgent;
import de.hawhamburg.reachability.sensor.SensorData;
import de.hawhamburg.reachability.tools.googleCalendar.enumeration.CalendarRequestType;
import de.hawhamburg.reachability.tools.googleCalendar.json.JsonGoogleCalendarRequest;
import de.hawhamburg.reachability.tools.googleCalendar.json.JsonGoogleCalendarResult;
import de.hawhamburg.reachability.util.ConnectionUtil;
import de.hawhamburg.reachability.util.OutputUtil;

/**  
 * Sensor to read events from the users Google(TM) calendar using the projects google calendar hook service via 
 * ActiveMQ.<br><br>
 * 
 * This sensor adapts the Google(TM) calendar by using the projects google calendar hook service via the ActiveMQ. It 
 * reads the next events and converts them into the internal data representation. This data will be provided to the 
 * reasoning agent via the AvtiveMQ.
 * 
 * @author  {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_AUTHOR} 
 * @version {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_VERSION}
 */
public class GoogleCalendarHookAdapter extends    Sensor
                                       implements SensorAdapter  {
    
    /**
     * Creates a new {@link GoogleCalendarHookAdapter} which provides the next events of the inhabitants Google(TM) 
     * calendar.<br><br>
     * 
     * The sensors configurations will be loaded from the XML file in  
     * {@link de.hawhamburg.reachability.sensor.Settings#SENSOR_CONFIGURATION_PATH}
     * which is named like this class.
     * 
     * @param dimension The {@link ReasonerDimension} for which this sensor is configured.
     */
    public GoogleCalendarHookAdapter(final ReasonerDimension dimension) {
        super(dimension);
    }

    /**
     * Provides the inhabitants next calendar events.
     * 
     * @return          The inhabitants next calendar events {@link SensorData}.
     */
    public final SensorData getSensorOutput() {
        JsonGoogleCalendarRequest req = this.sendRequest();

        Gson g = new Gson();
        String json = null;
        JsonGoogleCalendarResult msgObj = null;
        int counter = 0;
        int limit   = Integer.valueOf(this.getLocalConf("WAITING_RETRY_LIMIT"));
        
        while (json == null && counter++ < limit) {
            try {
                sleep(Long.valueOf(this.getLocalConf("WAITING_DELAY_TIME")));
            } catch (InterruptedException e) {
                OutputUtil.error("[InterruptedException] " + e.getMessage());
            }
            json = ConnectionUtil.getMessageOnTopic(this.getResultTopic());
            if (json != null) {
                msgObj = g.fromJson(json, JsonGoogleCalendarResult.class);
                if (!msgObj.requestId.equals(req.Id)) {
                    json   = null;
                    msgObj = null;
                }
            }
        }
        
        if (msgObj != null) {
            if (msgObj.amount == 1) {
                double start = msgObj.events.get(0).startTime.getValue();
                double end   = msgObj.events.get(0).endTime.getValue();
                    
                double[] data = {start, end};
                
                return new SensorData(this.getClass().getSimpleName(), this.getDataDimension(), data);
            } else {
                // No events found...
                return null;
            }
        } else {
            // No data found...
            return null;
        }
    }
    
    /**
     * Build a request for {@link de.hawhamburg.reachability.tools.googleCalendar.GoogleCalendarHookService} as
     * {@link JsonGoogleCalendarRequest}.
     * 
     * @return          The request as {@link JsonGoogleCalendarRequest}.
     */
    private JsonGoogleCalendarRequest sendRequest() {
        JsonGoogleCalendarRequest req = new JsonGoogleCalendarRequest();
        req.Id          = getID();
        req.Version     = ProjectInfo.REACHABILITY_AGENT_VERSION;
        req.requestType = CalendarRequestType.NEXT;
        req.amount      = 2;
        
        ConnectionUtil.setMessageOnTopic(this.getRequestTopic(), req);
        return req;
    }

    /**
     * Main method which starts this sensor adapter.
     * 
     * @param args      The start arguments as array of {@link String}.
     */
    public static void main(final String[] args) {
        // Remove all existing loggers
        Logger.getRootLogger().removeAllAppenders();
        
        // Build this sensor adapter by using the SensorAdapterFactory.
        SensorAdapter sa = SensorAdapterFactory.getConnectedSensorAdapter(
                                GoogleCalendarHookAdapter.class.getSimpleName(),
                                ReasonerDimension.calendar);

        // Start the sensor adapter as sensor agent, which supplied the required control functions.
        new SensorAgent(sa);
    }
}
