package de.hawhamburg.reachability.sensor;

import de.hawhamburg.reachability.reasoner.enumeration.ReasonerDimension;
import de.hawhamburg.reachability.StopableObject;

/**
 * Interface which needs to be implemented by each {@link SensorAdapter}.<br><br>
 * 
 * This interface defines the methods, which a {@link SensorAdapter} needs to implement to be used in common with the 
 * reachability project. Besides this interface, a {@link SensorAdapter} needs to implement the interface 
 * {@link StopableObject} as well.
 * 
 * @author  {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_AUTHOR} 
 * @version {@value de.hawhamburg.reachability.ProjectInfo#REACHABILITY_AGENT_VERSION}
 */
public interface SensorAdapter extends StopableObject {
    
    /**
     * Connects the sensor to its data source, for it'll be able to provide new sensor data in an internal data format
     * via the ActiveMQ.
     */
    void connect();
    
    /**
     * Requests the last measured sensor data from the {@link SensorAdapter} as {@link SensorData}.
     * 
     * @return      The last measured sensor data as {@link SensorData} object.
     */
    SensorData getSensorOutput();
    
    /**
     * Provides the name of the request topic for ActiveMQ communication, if the {@link SensorAdapter} is connected to 
     * its data source via the ActiveMQ and the data source needs requests to supply data, otherwise <code>null</code>.
     * 
     * @return      The name of the request topic for this {@link SensorAdapter}, if it's connected via ActiveMQ, 
     *              otherwise <code>null</code>.
     */
    String getRequestTopicName();
    
    /**
     * Provides the name of the result topic for ActiveMQ communication, if the {@link SensorAdapter} is connected to 
     * its data source via the ActiveMQ, otherwise <code>null</code>.
     * 
     * @return      The name of the result topic for this {@link SensorAdapter}, if it's connected via ActiveMQ, 
     *              otherwise <code>null</code>.
     */
    String getResultTopicName();

    /**
     * Provides the context dimension of the sensors data as {@link ReasonerDimension}.
     * 
     * @return      The {@link ReasonerDimension} of the sensors data.
     */
    ReasonerDimension getDataDimension();
    
    /**
     * A {@link SensorAdapter} extends the class {@link Thread}. To make the {@link Thread#start()} method visible, it 
     * needs to be included to this interface. 
     */
    void start();
    
    /**
     * Determines if the {@link SensorAdapter} has stopped working.
     * 
     * @return      <code>true</code> if the {@link SensorAdapter} has stopped working, otherwise <code>false</code>.
     */
    boolean stopped();
    
    /**
     * Sets the silent state. If silent is set to <code>true</code>, the worker won't publish his informations on the
     * internal data topic.
     * 
     * @param state The current state of silence as {@link Boolean}.
     */
    void setSilent(final boolean state);
}
